set :application, "chirkut_node.js"
set :repository, "git@github.com:directi/chirkut_deployment.git" 
set :scm, :git
set :user , "deploy" 
ssh_options[:forward_agent] = true
set :deploy_to, "/opt/chirkut/#{application}"
set :deploy_via, :rsync_with_remote_cache
set :use_sudo, false
set :keep_releases, 3 
set :branch, ENV['branch'] || 'release'

env = ENV['environment'] || 'staging'

if env == 'production'
  role :app, "xmpp1.chat.pw", "xmpp2.chat.pw"
else
  role :app, "ec2-50-19-180-58.compute-1.amazonaws.com"
end

namespace :deploy do 
  desc <<-DESC
  A macro-task that updates the code and fixes the symlink. 
  DESC
  task :default do 
    transaction do 
      update_code
      symlink
    end
  end
 
  task :update_code, :except => { :no_release => true } do 
    on_rollback { run "rm -rf #{release_path}; true" } 
    strategy.deploy! 
  end
 
  task :after_deploy do
    cleanup
  end
end

namespace :install do 
	task :post do
	  run "ln -s /opt/chirkut/#{application}/shared/node_modules /opt/chirkut/#{application}/current/node_modules"
		run "source ~/.bash_profile; cd /opt/chirkut/#{application}/current/; npm install"
		run "cd /opt/chirkut/#{application}/current/ && cp chirkut.conf.js.#{env} chirkut.conf.js"
	end 
end

namespace :app do	
	task :status do 
		sudo "/usr/bin/ruby /usr/bin/god status"
	end
	task :stop do 
		sudo "/usr/bin/ruby /usr/bin/god stop chirkut"
	end
	task :start do 	
		sudo "/usr/bin/ruby /usr/bin/god start chirkut"
	end 
end

namespace :god do 
	task :status do 
		sudo "/usr/bin/ruby /usr/bin/god status"
	end
	task :stop do 
		sudo "su - -c 'pgrep -f /usr/bin/god | xargs kill -9'" 
	end 
	task :start do 
		sudo "/usr/bin/ruby /usr/bin/god -c /opt/chirkut/#{application}/current/scripts/chirkut.god -P /var/run/god.pid -l /var/log/god.log" 
	end
end 

namespace :process do
	task :stop_all do
		run "pgrep -f 'chirkut|god' | sudo xargs kill -9" 
	end
end

namespace :app do 
	task :version do 
		run "source ~/.bash_profile; cd /opt/chirkut/#{application}/current/ && node run-server.js --version"
	end
end 

namespace :testBefore do
  task :before_deploy do
    run "mkdir -p /opt/chirkut/#{application}/shared/node_modules" 
    run "mkdir -p /opt/chirkut/#{application}/releases" 
  end
end
		
after "deploy" , "install:post"
before "deploy" , "testBefore:before_deploy"