/***
 * Monitors the time a packet took in the bosh server buffers.
 * Time between 'response' event of a packet and 'response-sent' event for
 * corresponding packet.
 */
var dutil = require('node-xmpp-bosh').dutil;

function log(msg){
    //TODO: Update this to write to a separate file
    dutil.log_it("INFO", msg);
}

function Monitor(boshServer){
    this.init(boshServer);
};

Monitor.prototype = {
    init: function(server){
        this.server = server;
        server.on('response', this.onResponseFromServer, 25);
        server.on('response-sent', this.onResponseSent);
        server.on('response-send-fail', this.onResponseSendFail);
        server.on('no-client', this.onClientDisconnect);
    },
    destroy: function(){
        this.server.removeListener('response', this.onResponseFromServer);
        this.server.removeListener('response-sent', this.onResponseSent);
        this.server.removeListener('response-send-fail', this.onResponseSendFail);
        this.server.removeListener('no-client', this.onClientDisconnect);
        
    },
    onResponseFromServer: function(xmlStanza, boshState){
        xmlStanza._inTime = new Date();
    },
    onResponseSent: function(boshResponse, boshState){
        var now = new Date();
        boshResponse.response.children.forEach(function(xmlStanza){
            if (xmlStanza._inTime) {
                log('RESPONSE-OUT:: ' + boshState.sid + ', ' + (now - xmlStanza._inTime));
            }
            else {
                log('MISSING-TIME:: ' + boshState.sid );
            }
        });        
    },
    onResponseSendFail: function(boshResponse, boshState){
    },
    onClientDisconnect: function(boshResponse, boshState){
        var now = new Date();
        boshResponse.response.children.forEach(function(xmlStanza){
            if (xmlStanza._inTime) {
                log('RESPONSE-DISCARD:: ' + boshState.sid + ', ' + (now - xmlStanza._inTime));
            }
        });
    }
}

var monitor;

exports.init = function(ctor_args){
    if (!ctor_args || !ctor_args.server) {
        throw new Error("You MUST get 'server' set to the BOSH server instance");
    }
    server = ctor_args.server;
    monitor = new Monitor(server);
};

exports.destroy = function(){
    monitor.destroy();
};
