var _ = require('underscore');
var BoosterClient = require(__dirname + '/booster_client').BoosterClient; 
 
var BoosterFilter = function(server){
    streams = {};
    server = server;

    var _isTalkToAuthStanza = function(stanza) {
        return stanza.name === "auth" && stanza.attrs['xmlns'] === 'http://talk.to/x-auth';
    };

    var _publishNode = function(sstate) {
        return function(stanza) {
            server.emit('nodes', [stanza], sstate);
        }
    };
    
    var _startBoosting = function(talktoStanzas, sstate) {
        var client = BoosterClient.create(talktoStanzas, _publishNode(sstate), sstate.to);
        _startSuppression(sstate.name, {
            client: client,
            extraResponses: [] 
        });
        client.once('done', _handleDone(sstate));
        client.once('stream-restart', _handleRestart(sstate));
    };
    
    var _handleRestart = function(sstate) {
        return function() {
            server.emit('stream-restart', sstate);
        }
    };
    
    var _handleDone = function(sstate) {
        return function(stanzas){
            var extraResponses = streams[sstate.name].extraResponses;
            _stopSuppression(sstate.name);
            _(stanzas).chain().concat(extraResponses).each(function(stanza){
                server.emit('response', stanza, sstate);
            });
        };
    };

    var _startSuppression = function(streamName, boostState) {
        streams[streamName] = boostState;
    };

    var _stopSuppression = function(streamName) {
        delete streams[streamName];
    };
    
    var _isSuppressed = function(sstate) {
        return sstate.name in streams;
    };
    
    var response = function(stanza, sstate) {
        if (!(_isSuppressed(sstate))) return;
        var streamInfo = streams[sstate.name];
        if (!streamInfo.client.consumes(stanza)) {
            streamInfo.extraResponses.push(stanza);
        }
        return false
    };
    
    var nodes = function(nodes, sstate) {
        if (_(nodes).any(_isTalkToAuthStanza)) {
            _startBoosting(nodes, sstate);
            return false;
        }
    };
    
    return {
        response: response,
        nodes: nodes
    };
}

module.exports = BoosterFilter;
