// -*- tab-width: 4 -*-

/* Directi Pvt. Ltd. 2011 */

// The redis plugin for chirkut.js


var nxb     = require('node-xmpp-bosh');
var dutil   = nxb.dutil;

// In our effort to be a 0-config application, we make our
// code a little dirty and very robust.
var redis = null;
var redis_host = null;
var redis_port = null;

try {
	redis = require('redis');
}
catch (ex) {
	dutil.log_it("WARN", "Chirkut.js::Could NOT load the 'redis' module");
}

// This is a monotically increasing 'packet ID' that is unique 
// to every packet delivered
var pid = 1;

var rclient = null;

// 
// The redis client we use buffers requests till a connection is
// established (also when it automatically reconnects to the redis
// server) to hide these *connection* related issues from the 
// clients. However, if we use this *feature*, then our memory
// usage will balloon beyond bounds. Hence, we stop publishing 
// messages if we are NOT connected to the redis server.
//
var redis_ready = false;

var server = null;


function handle_on_response(response, sstate) {
	// console.log("on_response:", response);

	// Check if we may send to redis
	if (!redis_ready) {
		return;
	}

	var pass_thru = false;
        var body = response.getChild('body') ;
        var hasNonEmptyBody = body && body.getText() !== '';


	if (!pass_thru && response.is('message') && hasNonEmptyBody &&
		(response.attrs.type == 'chat' || response.attrs.type == 'groupchat')) {
		pass_thru = true;
	}

	// Add your custom filters here and set 'pass_thru' accordingly

	if (!pass_thru) {
		return;
	}

	// We just add a unique ID to the response object
	response._pid = pid;
	pid += 1;

	rclient.publish(
		dutil.sprintf('/BOSH/PACKET/UNACK/%s/%s', sstate.state.sid, response._pid), 
		response.toString()
	);
}

function handle_on_response_acknowledged(wrapped_response, state) {
	// console.log("on_response_acknowledged:", wrapped_response);

	// Check if we may send to redis
	if (!redis_ready) {
		return;
	}

	wrapped_response.response.children.forEach(function(response) {
		// Check if we need to publish an ACK packet
		if (response._pid) {
			rclient.publish(dutil.sprintf('/BOSH/PACKET/ACK/%s/%s', state.sid, response._pid), '');
		}
	});
}


function init_redis() {
	if (!redis || rclient || redis_ready) {
		return;
	}

	rclient = redis.createClient(redis_port, redis_host);
	redis_ready = false;

	rclient.on('ready', function() {
		redis_ready = true;
	});

	rclient.on('error', function() {
		redis_ready = false;
	});

	server.on('response', handle_on_response, 30);
	server.on('response-acknowledged', handle_on_response_acknowledged);
}

function destroy_redis() {
	if (!redis || !rclient) {
		return;
	}

	server.removeListener('response', handle_on_response);
	server.removeListener('response-acknowledged', handle_on_response_acknowledged);

	rclient.quit();
	rclient = null;

}


exports.init = function(ctor_args) {
	if (!ctor_args || !ctor_args.server) {
		throw new Error("You MUST set 'server' to the BOSH server instance");
	}
    
	server = ctor_args.server;
    redis_host = ctor_args.config.redis_host || '127.0.0.1';
    redist_port = ctor_args.config.redis_port || 6379;

	init_redis();
};

exports.destroy = function() {
	destroy_redis();
};
