/* 
   adds timestamps to the message packets that 
   do not have a timestamp on them in TZ format.

   attribute: stimestamp
   timeformat: TZ
*/

var server = null;

var add_timestamp = function (response, sstate) {
    if (!response.is('message')) {
        return;
    }
    var body = response.getChild('body') ;
    var is_non_empty_message = body && body.getText() !== '';

    if (is_non_empty_message && !response.attrs.stimestamp) {
        var now = new Date();

        var prepend_zero_if_reqd = function (n) {
            if (n < 10) {
                return '0' + n;
            }
            return '' + n;
        };

        var date = now.getUTCFullYear() + '-' + prepend_zero_if_reqd(now.getUTCMonth() + 1) + '-' + prepend_zero_if_reqd(now.getUTCDate());
        var time = prepend_zero_if_reqd(now.getUTCHours()) + ':' + prepend_zero_if_reqd(now.getUTCMinutes()) + ':' + prepend_zero_if_reqd(now.getUTCSeconds());
        
        response.attrs.stimestamp = date + 'T' + time + 'Z';
    }
};

var init = function () {
    if (!server) {
        return;
    }
    /* 
       don't need to do anything about the 
       order of listeners since the response
       is sent on nextTick
    */
    server.on('response', add_timestamp);
};

var destroy = function () {
    if (!server) {
        return;
    }
    server.removeListener('response', add_timestamp);
};

exports.init = function(ctor_args) {
	if (!ctor_args || !ctor_args.server) {
		throw new Error("You MUST set 'server' to the BOSH server instance");
	}
	server = ctor_args.server;

	init();
};

exports.destroy = function() {
	destroy();
};
