#!/usr/bin/env node
// -*- tab-width: 4 -*-

/* Directi Pvt. Ltd. 2011 */

/* Please keep this file as ONE file and resist your urge
 * to *refactor* it into multiple files. Keep it simple...
 *
 */

var path    = require('path');
var webrepl = require('webrepl');
var emailjs = require('emailjs');
var ltx     = require('ltx');
var us      = require('underscore');
var nxb     = require('node-xmpp-bosh');
var os      = require('os');
var util    = require('util');
var dutil   = nxb.dutil;




var SEND_CRASH_MAIL_TO = [ ].join(', ');

var SEND_CRASH_MAIL_FROM = 'BOSH Report<noreply@github.com>';

var CRASH_MAIL_SUBJECT = 'BOSH server v%s at http://%s:%s%s crashed';

var SMTP_HOST     = 'smtp.directi.com';
var SMTP_PORT     = 25;
var SMTP_USERNAME = 'boshserver@directi.com';
var SMTP_PASSWORD = 'lB@BHjp2';

var BOSH_DEFAULT_CONFIG_PATH = './chirkut.conf.js';

var server_options = { };


function unload_plugin(plugin_path) {
	// Unloads a (possibly loaded) plugin

	plugin_path = require.resolve(plugin_path);

	var prev = dutil.find_module(plugin_path);
	if (prev.handle && typeof prev.handle.destroy == 'function') {
		prev.handle.destroy();
	}
	return !!prev.handle;
}

function load_plugin(plugin_path, ctor_args) {
	// Dynamically loads/unloads a plugin
	unload_plugin(plugin_path);

	plugin_path = require.resolve(plugin_path);

	var curr = dutil.require_again(plugin_path);
	if (curr && typeof curr.init == 'function') {
		curr.init(ctor_args);
	}

	return curr;
}

function send_email(from, to, subject, text, cb) {
	if (!SEND_CRASH_MAIL_TO) {
		return;
	}

    var server = emailjs.server.connect({
        host: SMTP_HOST,
        port: SMTP_PORT,
        username: SMTP_USERNAME,
        password: SMTP_PASSWORD
    });

    var message = emailjs.message.create({
        to: to,
        from: from,
        subject: subject,
        text: text
    });

    server.send(message, function(err, result) {
        // console.error('Mailer callback:', err, result);
        cb(err, message);
    });
}

function app_meta() {
	var fs = require('fs');
	var package_json_path = require.resolve("./package.json");

	var pkg_str = fs.readFileSync(package_json_path);
	var meta = JSON.parse(pkg_str);
	return meta;
}


function get_version() {
	return app_meta().version;
}

function show_version() {
	var meta = app_meta();
	console.log(meta.name + ": BOSH server version " + meta.version);
}


process.on('uncaughtException', function(ex) {
	console.error("Uncaught Exception:", ex.toString());
	console.error("Stack trace:\n" + ex.stack);

	var text = 'Do Not Pass Go, Do Not Collect $200\n\n' + 
		'server_options:\n' + util.inspect(server_options, false, 4) + 
		'\n\nHere is the stack trace:\n\n' + ex.stack;

	console.error('Sending email:', text);
	var subject = dutil.sprintf(CRASH_MAIL_SUBJECT, get_version(), os.hostname(), server_options.port, server_options.path);
	send_email(SEND_CRASH_MAIL_FROM, SEND_CRASH_MAIL_TO, subject, text, function() {
		process.exit(2);
	});
});


function main() {
	var opts = require('tav').set({
		logging: {
			note: "The logging level to use (default: DEBUG)",
			value: -1
		},
		path: {
			note: "The HTTP PATH at which to run the BOSH server (default: /http-bind/)",
			value: -1
		},
		port: {
			note: "The port on which to run the BOSH server (default: 5280)",
			value: -1
		},
		host: {
			note: "The host on which to the BOSH server should listen for connections (default: 0.0.0.0)", 
			value: -1
		}, 
		version: {
			note: "Display version info and exit",
			value: false
		}, 
        redis_host: {
            note: "The host on which the redis server is running. This is required for redis plugin which pushes messages to the push-handler.",
            value: -1
        },
        redis_port: {
            note: "The port on above host where the redis server is running.",
            value: -1
        },
		webrepl: {
			note: "The port to run the webrepl on (default: 9090). Set to 65536 to disable the webrepl.", 
			value: -1
		}, 
		config: {
			note: "The config file to load (default: /etc/bosh.js.conf)", 
			value: BOSH_DEFAULT_CONFIG_PATH
		}
	}, "Usage: chirkut.js [option=value]");

	if (opts.version) {
		show_version();
		return;
	}

	if (opts.config) {
		if (opts.config[0] != '/') {
			opts.config = "./" + opts.config;
		}

		try {
			var _cfg = require(opts.config);
			server_options = _cfg.config;
		}
		catch(ex) {
			if (opts.config != BOSH_DEFAULT_CONFIG_PATH) {
				var estr = ex.toString() + ": while trying to read " + 
					"config file '" + opts.config + "'";
				throw new Error(estr);
			}
		}
	}

	if (opts.port == -1) {
		if (!server_options.port) {
			server_options.port = 5280;
		}
	}
	else {
		var _port = Math.ceil(opts.port);
		if (!_port) {
			_port = 5280;
		}
		server_options.port = _port;
	}

	if (opts.host == -1) {
		if (!server_options.host) {
			server_options.host = '0.0.0.0';
		}
	}
	else {
		server_options.host = opts.host;
	}

	if (opts.path == -1) {
		if (!server_options.path) {
			server_options.path = '/http-bind/';
		}
	}
	else {
		if (opts.path.length > 0 && opts.path[0] != "/") {
			opts.path = "/" + opts.path;
		}
		server_options.path = opts.path;
	}

	if (opts.logging == -1) {
		if (!server_options.logging) {
			server_options.logging = 'DEBUG';
		}
	}
	else {
		server_options.logging = opts.logging.toUpperCase();
	}

	if (opts.webrepl == -1) {
		if (!server_options.webrepl) {
			server_options.webrepl = 9090;
		}
	}
	else {
		server_options.webrepl = Math.floor(opts.webrepl);
	}

    if (opts.redis_host == -1) {
        if (!server_options.redis_host) {
            server_options.redis_host = '127.0.0.1';
        }
    } else {
        server_options.redis_host = opts.redis_host;
    }
    
    if (opts.redis_port == -1) {
        if (!server_options.redis_port) {
            server_options.redis_port = 6379;
        }
    } else {
        server_options.redis_port = opts.redis_port;
    }

	if (server_options.crash_email_recipients instanceof Array) {
		SEND_CRASH_MAIL_TO = server_options.crash_email_recipients.join(', ');
	}


	var start_time = new Date();

	var msg = nxb.dutil.sprintf("Starting the BOSH server on 'http://%s:%s%s' at '%s'", 
								server_options.host, server_options.port, server_options.path, start_time);
	var hr  = "+-" + nxb.dutil.repeat('-', msg.length).join('') + "-+";
	console.log(hr);
	console.log("| " + msg + " |");
	console.log(hr);

	var repl = null;

	// First start the webrepl (if it is asked for)
	if (server_options.webrepl < 65536) {
		repl   = webrepl.start(server_options.webrepl);

		repl.context.nxb    = nxb;

		// uptime
		repl.context.__defineGetter__('uptime', function() {
			return nxb.dutil.time_diff(start_time, new Date());
		});

		// For some reason, webrepl complains if we don't have a setter
		// defined
		repl.context.__defineSetter__('uptime', function() { });

		// version
		repl.context.__defineGetter__('version', function() {
			return get_version();
		});

		// For some reason, webrepl complains if we don't have a setter
		// defined
		repl.context.__defineSetter__('version', function() { });

		repl.context.unload_plugin = unload_plugin;
		repl.context.load_plugin = load_plugin;
		repl.context.self = repl;

		repl.context.help = 'Please visit ' + 
			'<a href="https://github.com/directi/chirkut_deployment">https://github.com/directi/chirkut_deployment</a>' +
			' for more information';
	}

	var server = nxb.start_bosh(server_options);

	if (repl) {
		repl.context.server = server;
	}

	// Load all plugins. Do NOT catch exceptions. Let them crash the server.
	// Serves all lousy developers well for writing bad code.
	if (server_options.plugins instanceof Array) {
		server_options.plugins.forEach(function(plugin_path) {
			load_plugin(plugin_path, {
				server : server, 
				webrepl: repl, 
				nxb    : nxb,
                config : server_options
			});
		});
	}

}

// Go!! We obviously can't get over the C-effect!!
main();
