require(__dirname + '/../test_utils');
var assert = require('assert');
var _ = require('underscore'); 
var states = require(__dirname + '/../../plugins/login_booster/states');

var trueFn = function() {return true};
var falseFn = function() {return false};
var undefinedFn = function(){};

var popfn = function(arr) {
    return function() {
        return arr.shift()
    }
}

module.exports = {
    'it is not done if any state is not done': function() {
        var state_1 = {done: trueFn};
        var state_2 = {done: falseFn};
        var state_3 = {done: trueFn};
        var state = new states.Composite([state_1, state_2, state_3]);
        assert.isFalse(state.done());
    },
    
    'it is done when all the states are done': function() {
        var state_1 = {done: trueFn};
        var state_2 = {done: trueFn};
        var state_3 = {done: trueFn};
        var state = new states.Composite([state_1, state_2, state_3]);        
        assert.isTrue(state.done());
    },
    
    'popStanza should pop one by one from the first state, then the next until all states return undefined and then return undefined (ignoring done states)': function() {
        var state_1 = {popStanza: popfn(["one","two"]), done: falseFn};
        var state_2 = {popStanza: popfn([]), done: falseFn};
        var state_3 = {popStanza: popfn(["seven"]), done: falseFn};
        var state_4 = {popStanza: popfn(["twenty"]), done: trueFn};
        var state = new states.Composite([state_1, state_2, state_3, state_4]);
        assert.equal("one", state.popStanza());
        assert.equal("two", state.popStanza());
        assert.equal("seven", state.popStanza());
        assert.isUndefined(state.popStanza());
    },

	'it should restart the stream if any state requires a restart': function(){
	    var state_1 = {shouldRestartStream: trueFn, done: falseFn};
        var state_2 = {shouldRestartStream: falseFn, done: falseFn};
        var state_3 = {shouldRestartStream: falseFn, done: falseFn};
        var state = new states.Composite([state_1, state_2, state_3]);
        assert.isTrue(state.shouldRestartStream());
	},

	'it should restart not the stream if no state requires a restart': function(){
	    var state_1 = {shouldRestartStream: falseFn, done: falseFn};
        var state_2 = {shouldRestartStream: falseFn, done: falseFn};
        var state_3 = {shouldRestartStream: falseFn, done: falseFn};
        var state_4 = {shouldRestartStream: trueFn, done: trueFn};
        var state = new states.Composite([state_1, state_2, state_3, state_4]);
        assert.isFalse(state.shouldRestartStream());
	},
	
	'it does not changeUsing a stanza if none of the states change using the stanza (ignoring done states)': function(){
	    var state_1 = {changeUsing: undefinedFn, done: falseFn};
        var state_2 = {changeUsing: undefinedFn, done: falseFn};
        var state_3 = {changeUsing: undefinedFn, done: falseFn};
        var state_4 = {changeUsing: trueFn, done: trueFn};
        var state = new states.Composite([state_1, state_2, state_3, state_4]);
        assert.isUndefined(state.changeUsing());
	},
	
	'it changes to a new composite state with all the old states that did not change and the new state that changed using the stanza': function() {
	    var stanza = "some response"
	    var new_state = {};
	    var state_1 = {changeUsing: undefinedFn, done: falseFn};
        var state_2 = {changeUsing: function(response){if (response === stanza) return new_state}, done: falseFn};
        var state_3 = {changeUsing: undefinedFn, done: falseFn};
        var state = new states.Composite([state_1, state_2, state_3]);
        var result = state.changeUsing(stanza)
        assert.equal('Composite', result.name);
        assert.deepEqual([state_1, new_state, state_3], result._states);
	},
	
	'when responseStanzas is invoked, it provides a list of all the responseStanzas of all the states': function() {
	    var state_1 = {responseStanzas: function(){return ["s1", "s2"]}};
        var state_2 = {responseStanzas: function(){return ["s3"]}};;
        var state_3 = {responseStanzas: function(){return ["s4", "s5"]}};
        var state = new states.Composite([state_1, state_2, state_3]);
        assert.deepEqual(["s1", "s2", "s3", "s4", "s5"], state.responseStanzas());
	},
	
	'it does not invoke changeUsing on a done state': _.to_be_implemented
}