require(__dirname + '/../test_utils');
var assert = require('assert');
var _ = require('underscore'); 
var states = require(__dirname + '/../../plugins/login_booster/states');

var preauth_state = function() {
    return new states.PreAuth({input: _('talkto_auth').createNodes()});
} 

module.exports = {
    'it sends a PLAIN_PW_AUTH packet using the credentials given': function() {
        assert.stanzaEqual({
            'auth': {
                '@xmlns': 'urn:ietf:params:xml:ns:xmpp-sasl',
                '@mechanism': 'PLAIN-PW-TOKEN',
                '#text': 'Zm9vAABiYXIAYmF6' 
            }
        }, preauth_state().popStanza());
    },
    
    'it sends only one packet': function() {
        var state = preauth_state();
        state.popStanza();
        assert.equal(undefined, state.popStanza());
    },

    'it switches to a Composite state composed of AuthSuccess and PreRestart state if the authentication succeeds giving it the response from the server': function() {
        var auth_success = _('auth_success').createStanza();
        var state = preauth_state();
        var newState = state.changeUsing(auth_success);
        assert.equal('Composite', newState.name);
        assert.equal('AuthSuccess', newState._states[0].name);
        assert.equal('PreRestart', newState._states[1].name);
        assert.equal(auth_success, state._stateData['auth']);
        assert.equal('foo', state._stateData['jid']);
    },

    'it switches to a AuthFail state if the authentication fails giving it the response from the server': function() {
        var auth_failure = _('auth_failure').createStanza();
        var newState = preauth_state().changeUsing(auth_failure);
        assert.equal('AuthFail', newState.name);
        assert.equal(auth_failure, newState._stateData['auth']);
    },

	'it does not return a new state if the stanza from the server is not an auth success or failure': function() {
		var newState = preauth_state().changeUsing(_('dummy').createStanza());
		assert.isUndefined(newState);
	},
	
	'it should not restart stream' : function() {
		assert.equal(false, preauth_state().shouldRestartStream());
	},

    'it is not done': function() {
       assert.equal(false, preauth_state().done()); 
    }
}