require(__dirname + '/../test_utils');
var assert = require('assert');
var _ = require('underscore'); 
var states = require(__dirname + '/../../plugins/login_booster/states');

function preBindState(input, stream_features) {
    stream_features = stream_features || 'stream_features_with_bind';
    return new states.PreBind({
        input: _(input).createNodes(),
        stream_features: _(stream_features).createStanza()
    });
}

module.exports = {
    'it sends an empty bind packet with no resource identifier if the client has not requested one': function() {
        assert.stanzaEqual({iq: {
            '@type': 'set',
            '@id': 'lb_bind_1',
            bind: {
                '@xmlns': 'urn:ietf:params:xml:ns:xmpp-bind',
                '#text': ''
            }}
        }, preBindState('talkto_auth').popStanza());
    },
    'it sends a bind packet with the requested resource id': function() {
        assert.stanzaEqual({iq: {
            '@type': 'set',
            '@id': 'lb_bind_1',
            bind: {
                '@xmlns': 'urn:ietf:params:xml:ns:xmpp-bind',
                'resource': {
                    '#text': 'my_awesome_client'
                }
            }}
        }, preBindState('talkto_auth_with_resource').popStanza()); 
    },
    
    'it sends only one packet': function() {
        var state = preBindState('talkto_auth');
        state.popStanza();
        assert.equal(undefined, state.popStanza());
    },

    'on successful bind, it switches to a Composite state including PreSession state and BindComplete state if the server supports sessions': function() {
        var successful_bind = _.createStanza('successful_bind');
        var state = preBindState('talkto_auth', 'stream_features_with_session');
        var newState = state.changeUsing(successful_bind);
        assert.equal('Composite', newState.name);
        assert.equal('BindComplete', newState._states[0].name);
        assert.equal('PreSession', newState._states[1].name);
        assert.equal(successful_bind, state._stateData['bind']);
    },
    
    'on successful bind it stores in the state against jid the jid along with the resource id': function() {
        var successful_bind = _.createStanza('successful_bind');
        var state = preBindState('talkto_auth', 'stream_features_with_session');
        state.changeUsing(successful_bind);
        assert.equal('juliet@example.com/balcony', state._stateData['jid']);
    },
    
    'on successful bind, it switches to a PreRoster state if the stream features does not include a session': function() {
        var successful_bind = _.createStanza('successful_bind');
        var state = preBindState('talkto_auth', 'stream_features_with_bind');
        var newState = state.changeUsing(successful_bind);
        assert.equal('Composite', newState.name);
        assert.equal('BindComplete', newState._states[0].name);
        assert.equal('PreRoster', newState._states[1].name);
        assert.equal(successful_bind, state._stateData['bind']);
    },

	'it should not restart stream': function() {
		assert.isFalse(preBindState('talkto_auth').shouldRestartStream());
	}, 

   'it is not done': function() {
       assert.isFalse(preBindState('talkto_auth').done()); 
    },

	'it does not return a new state for any other stanza from the server': function() {
		var newState = preBindState('talkto_auth').changeUsing(_('dummy_iq').createStanza());
		assert.isUndefined(newState);
	},

    'it  whines if there is no resource identifier in the resource element': _.to_be_implemented,
    'it does something if the resource identifier cannot be processed': _.to_be_implemented,
    'it does something if the not allowed to bind a resource': _.to_be_implemented,
    'it does something if the resource identifier is in use': _.to_be_implemented
}