require(__dirname + '/../test_utils');
var assert = require('assert');
var _ = require('underscore'); 
var states = require(__dirname + '/../../plugins/login_booster/states');

function preDomainDiscoState() {
    return new states.PreDomainItemsDisco({
        'jid': 'juliet@example.com/balcony',
        'to':'shakespeare.lit'
    });
}

module.exports = {
    'it sends a domain disco#items': function() {
        var state = preDomainDiscoState();
        assert.stanzaEqual({'iq': {
            '@from': 'juliet@example.com/balcony',
            "@id": 'lb_domain_items_1',
            '@to': 'shakespeare.lit',
            '@type': 'get',
            'query': {
                '@xmlns': 'http://jabber.org/protocol/disco#items'
            }}},state.popStanza());
    },
    
    'it sends only one packet': function() {
        var state = preDomainDiscoState();
        state.popStanza();
        assert.equal(undefined, state.popStanza());
    },
    
    'it switches to a preServiceDiscoInfo state given the items': function() {
        var newState = preDomainDiscoState().changeUsing(_.createStanza('domain_disco_items_result'));
        assert.equal('PreServiceInfoDisco', newState.name);
    },
    
 	'it should not restart stream': function() {
 		assert.isFalse(preDomainDiscoState().shouldRestartStream());
 	}, 

    'it is not done': function() {
        assert.isFalse(preDomainDiscoState().done()); 
     },

 	'it does not return a new state for any other stanza from the server': function() {
 		var newState = preDomainDiscoState().changeUsing(_('dummy_iq').createStanza());
 		assert.isUndefined(newState);
 	},
 	
 	'it does something if there is an empty list': _.to_be_implemented
}