require(__dirname + '/../test_utils');
var assert = require('assert');
var _ = require('underscore'); 
var states = require(__dirname + '/../../plugins/login_booster/states');

function preRosterState(input) {
    return new states.PreRoster({
        'jid': 'juliet@example.com/balcony',
        'input': _(input || 'talkto_auth').createNodes()
    });
}

module.exports = {
    'it sends a roster get': function() {
        var state = preRosterState();
        assert.stanzaEqual({'iq': {
            '@from': 'juliet@example.com/balcony',
            '@type': 'get',
            '@id': 'lb_roster_get_1',
            'query': {
                '@xmlns': 'jabber:iq:roster'
            }}
        },state.popStanza());
    },
    
    'it sends only one packet': function() {
        var state = preRosterState();
        state.popStanza();
        assert.equal(undefined, state.popStanza());
    },
    
    'on recieving the roster, it stores the response and switches to a RosterFetched State if there is no vcard requested': function() {
        var roster = _.createStanza('roster_response');
        var newState = preRosterState().changeUsing(roster);
        assert.equal('RosterFetched', newState.name);
        assert.equal(roster, newState._stateData['roster']);
    },
    
    'on recieving the roster, it stores the response and switches to a composite state containing the RosterFetched state AND a PreVCard state if vcards were requested': function() {
        var roster = _.createStanza('roster_response');
        var newState = preRosterState('talkto_auth_with_vcard').changeUsing(roster);
        assert.equal('Composite', newState.name);
        assert.equal('RosterFetched', newState._states[0].name);
        assert.equal('PreVCard', newState._states[1].name);
        assert.equal(roster, newState._states[0]._stateData['roster']);
    },
    
 	'it should not restart stream': function() {
 		assert.isFalse(preRosterState().shouldRestartStream());
 	}, 

    'it is not done': function() {
        assert.isFalse(preRosterState().done()); 
     },

 	'it does not return a new state for any other stanza from the server': function() {
 		var newState = preRosterState().changeUsing(_('dummy_iq').createStanza());
 		assert.isUndefined(newState);
 	},
 	
 	'it does something if there is no roster': _.to_be_implemented
}