require(__dirname + '/../test_utils');
var assert = require('assert');
var _ = require('underscore'); 
var states = require(__dirname + '/../../plugins/login_booster/states');

function preServiceDiscoState() {
    return new states.PreServiceInfoDisco({
        'jid': 'juliet@example.com/balcony',
        'domain_items': _.createStanza('domain_disco_items_result')
    });
}

module.exports = {
    'it sends a service disco#info for each service and no more': function() {
        var state = preServiceDiscoState();
        assert.stanzaEqual({'iq': {
            '@from': 'juliet@example.com/balcony',
            "@id": 'lb_service_disco_1',
            '@to': 'people.shakespeare.lit',
            '@type': 'get',
            'query': {
                '@xmlns': 'http://jabber.org/protocol/disco#info'
            }}},state.popStanza());
        assert.stanzaEqual({'iq': {
            '@from': 'juliet@example.com/balcony',
            "@id": 'lb_service_disco_2',
            '@to': 'plays.shakespeare.lit',
            '@type': 'get',
            'query': {
                '@xmlns': 'http://jabber.org/protocol/disco#info'
            }}},state.popStanza());
        assert.stanzaEqual({'iq': {
            '@from': 'juliet@example.com/balcony',
            "@id": 'lb_service_disco_3',
            '@to': 'chat.shakespeare.lit',
            '@type': 'get',
            'query': {
                '@xmlns': 'http://jabber.org/protocol/disco#info'
            }}},state.popStanza());
        assert.isUndefined(state.popStanza());    
    },
    
    'it switches to itself for a non-muc feature response stanza': function() {
        var state = preServiceDiscoState();
        assert.equal(state, state.changeUsing(_.createStanza('non_muc_service_disco_response_1')));
    },
    
    'it switches to a Composite containing itself and a PreServiceItemsDisco state for any muc feature response': function() {
        var state = preServiceDiscoState();
        var newState = state.changeUsing(_.createStanza('plays_muc_service_disco_response'));
        assert.equal('Composite', newState.name);
        assert.equal('PreServiceItemsDisco', newState._states[0].name);
        assert.equal('plays.shakespeare.lit', newState._states[0]._serviceName);
        assert.equal(state, newState._states[1]);
    },
    
    'it treats duplicate responses as non muc responses': function() {
        var state = preServiceDiscoState();
        state.changeUsing(_.createStanza('plays_muc_service_disco_response'));
        var newState = state.changeUsing(_.createStanza('plays_muc_service_disco_response'));
        assert.equal(state, newState);
    },
    
    'if all the service info responses have arrived, it switches to a Dummy state': function() {
        var state = preServiceDiscoState();
        state.changeUsing(_.createStanza('non_muc_service_disco_response_1'));
        state.changeUsing(_.createStanza('non_muc_service_disco_response_2'));
        var newState = state.changeUsing(_.createStanza('non_muc_service_disco_response_3'));
        assert.equal(states.Dummy, newState);
        
        var state = preServiceDiscoState();
        state.changeUsing(_.createStanza('non_muc_service_disco_response_1'));
        state.changeUsing(_.createStanza('non_muc_service_disco_response_2'));
        var newState = state.changeUsing(_.createStanza('general_muc_service_disco_response'));
        assert.equal('Composite', newState.name);
        assert.equal('PreServiceItemsDisco', newState._states[0].name);
        assert.equal('chat.shakespeare.lit', newState._states[0]._serviceName);
        assert.equal(states.Dummy, newState._states[1]);
    },
    
 	'it should not restart stream': function() {
 		assert.isFalse(preServiceDiscoState().shouldRestartStream());
 	}, 

    'it is not done': function() {
        assert.isFalse(preServiceDiscoState().done()); 
     },

 	'it does not return a new state for any other stanza from the server': function() {
 		var newState = preServiceDiscoState().changeUsing(_('dummy_iq').createStanza());
 		assert.isUndefined(newState);
 	},
 	
 	'it does something if a reply never comes back': _.to_be_implemented
 }