require(__dirname + '/../test_utils');
var assert = require('assert');
var _ = require('underscore'); 
var states = require(__dirname + '/../../plugins/login_booster/states');

function preServiceItemsDiscoState() {
    return new states.PreServiceItemsDisco({
        'jid': 'juliet@example.com/balcony'
    }, 'chat.shakespeare.lit');
}

module.exports = {
    'it sends a service disco#items to the service': function() {
        var state = preServiceItemsDiscoState();
        assert.stanzaEqual({'iq': {
            '@from': 'juliet@example.com/balcony',
            "@id": 'lb_service_items_chat.shakespeare.lit',
            '@to': 'chat.shakespeare.lit',
            '@type': 'get',
            'query': {
                '@xmlns': 'http://jabber.org/protocol/disco#items'
            }}},state.popStanza());
        assert.isUndefined(state.popStanza());    
    },
    
    'it sends only one packet': function() {
        var state = preServiceItemsDiscoState();
        state.popStanza();
        assert.equal(undefined, state.popStanza());
    },
        
    'it switches to a rooms fetched state upon recieving the items response stanza': function() {
        var response = _.createStanza('chat_service_items_disco_response');
        var newState = preServiceItemsDiscoState().changeUsing(response);
        assert.equal('RoomsFetched', newState.name);
        assert.equal(response, newState._stateData['rooms']['chat.shakespeare.lit']);
    },
    
    'it does not return a new state for any items response for a different item from the server': function() {
 		var newState = preServiceItemsDiscoState().changeUsing(_('plays_service_items_disco_response').
 		createStanza());
 		assert.isUndefined(newState);
 	},
 	
    'it should not restart stream': function() {
 		assert.isFalse(preServiceItemsDiscoState().shouldRestartStream());
 	}, 

    'it is not done': function() {
        assert.isFalse(preServiceItemsDiscoState().done()); 
     },

 	'it does not return a new state for any other stanza from the server': function() {
 		var newState = preServiceItemsDiscoState().changeUsing(_('dummy_iq').createStanza());
 		assert.isUndefined(newState);
 	},
 	
 	'it does something if a reply never comes back': _.to_be_implemented,
 	
 	'it does something in case the reply is a partial list': _.to_be_implemented
 }