require(__dirname + '/../test_utils');
var assert = require('assert');
var _ = require('underscore'); 
var states = require(__dirname + '/../../plugins/login_booster/states');

function preSessionState() {
    return new states.PreSession({'to':'example.com'});
}

module.exports = {
    'it sends a session creation stanza': function() {
        assert.stanzaEqual({iq: {
            '@to': 'example.com',
            '@type': 'set',
			'@id': 'lb_sess_1',
            session: {
                '@xmlns': 'urn:ietf:params:xml:ns:xmpp-session'
            }}
        }, preSessionState().popStanza());
    },
    
    'it sends only one packet': function() {
        var state = preSessionState();
        state.popStanza();
        assert.equal(undefined, state.popStanza());
    },

    'on successful session creation, it switches to a PreRoster state': function() {
        var successful_session = _.createStanza('successful_session');
        var newState = preSessionState().changeUsing(successful_session);
        assert.equal('PreRoster', newState.name);
    },

	'it should not restart stream': function() {
		assert.isFalse(preSessionState().shouldRestartStream());
	}, 

   'it is not done': function() {
       assert.isFalse(preSessionState().done()); 
    },

	'it does not return a new state for any other stanza from the server': function() {
		var newState = preSessionState().changeUsing(_('dummy').createStanza());
		assert.isUndefined(newState);
	},


    'it does something if the server returns an internal server error': _.to_be_implemented,
    'it does something if the server responds with forbidden': _.to_be_implemented,
    'it does something if the server sends conflict': _.to_be_implemented

}