require(__dirname + '/../test_utils');
var assert = require('assert');
var _ = require('underscore'); 
var states = require(__dirname + '/../../plugins/login_booster/states');

function preVCardState() {
    return new states.PreVCard({
        'roster': _.createStanza('roster_response'),
        'jid': 'juliet@example.com/balcony'
    });
}

module.exports = {
    'it sends a vcard get for each jid in the roster and no more': function() {
        var state = preVCardState();
        assert.stanzaEqual({'iq': {
            '@from': 'juliet@example.com/balcony',
            "@id": 'lb_vcard_get_4',
            '@to': 'foo@bar.baz',
            '@type': 'get',
            'vCard': {
                '@xmlns': 'vcard-temp'
            }}},state.popStanza());
        assert.stanzaEqual({'iq': {
            '@from': 'juliet@example.com/balcony',
            "@id": 'lb_vcard_get_3',
            '@to': 'benvolio@example.net',
            '@type': 'get',
            'vCard': {
                '@xmlns': 'vcard-temp'
            }}},state.popStanza());
        assert.stanzaEqual({'iq': {
            '@from': 'juliet@example.com/balcony',
            "@id": 'lb_vcard_get_2',
            '@to': 'mercutio@example.com',
            '@type': 'get',
            'vCard': {
                '@xmlns': 'vcard-temp'
            }}},state.popStanza());
        assert.stanzaEqual({'iq': {
            '@from': 'juliet@example.com/balcony',
            "@id": 'lb_vcard_get_1',
            '@to': 'romeo@example.net',
            '@type': 'get',
            'vCard': {
                '@xmlns': 'vcard-temp'
            }}},state.popStanza());
        assert.isUndefined(state.popStanza());    
    },
    
    'it switches to itself for any vcard response and stores the response': function() {
        var state = preVCardState();
        var response = _('romeo_vcard').createStanza();
        var newState = state.changeUsing(response);
        assert.equal(state, newState);
        assert.equal(response, state._stateData['vcards']['romeo@example.net']);
    },
    
    'it switches to itself for an error vcard response': function() {
        var state = preVCardState();
        var response = _('benvolio_vcard_error').createStanza();
        var newState = state.changeUsing(response);
        assert.equal(state, newState);
        assert.equal(response, state._stateData['vcards']['benvolio@example.net']);
    },
    
    'it swallows duplicates and returns itself' : function() {
        var state = preVCardState();
        var vcard_response = _('benvolio_vcard').createStanza(); 
        state.changeUsing(vcard_response);
        var newState = state.changeUsing(_('benvolio_vcard_error').createStanza());
        assert.equal(state, newState);
        assert.equal(vcard_response, state._stateData['vcards']['benvolio@example.net']);
    },
    
    'it switches to a Dummy state when the last response is recieved': function() {
        var state = preVCardState();
        state.changeUsing(_('benvolio_vcard_error').createStanza());
        state.changeUsing(_('romeo_vcard').createStanza());
        state.changeUsing(_('mercutio_vcard').createStanza());
        var newState = state.changeUsing(_('foo_vcard').createStanza());
        assert.equal(states.Dummy, newState);
    },
    
 	'it should not restart stream': function() {
 		assert.isFalse(preVCardState().shouldRestartStream());
 	}, 

    'it is not done': function() {
        assert.isFalse(preVCardState().done()); 
     },

 	'it does not return a new state for any other stanza from the server': function() {
 		var newState = preVCardState().changeUsing(_('dummy_iq').createStanza());
 		assert.isUndefined(newState);
 	},
 	
 	'it does something if a reply never comes back': _.to_be_implemented
 }