require(__dirname + '/../test_utils');
var assert = require('assert');
var _ = require('underscore'); 
var states = require(__dirname + '/../../plugins/login_booster/states');

var rosterFetchedState = function() {
    return new states.RosterFetched({
        roster: _.createStanza('roster_response'),
        jid: 'juliet@example.com/chamber'
    });
}

module.exports = {
    'it is done': function() {
        assert.isTrue(rosterFetchedState().done());
    },

    'popStanza should return undefined': function() {
        assert.isUndefined(rosterFetchedState().popStanza());
    },

    'it should not restart stream' : function() {
        assert.equal(false, rosterFetchedState().shouldRestartStream());
    },

    'it sends a talk-to roster': function() {
        var response = rosterFetchedState().responseStanzas();
        assert.equal(1, response.length);
        var stanza = _(response).head();
        assert.stanzaEqual({
            'iq': {
                '@type': 'set',
                '@id': 'lb_roster_1',
                '@to': 'juliet@example.com/chamber',
                'query': {
                    '@xmlns': 'http://talk.to/x-roster',
                    '#children': [
                    {'item': {
                        '@jid': 'romeo@example.net',
                        '@name': 'Romeo',
                        '@subscription': 'both',
                        'group': {
                            '#text': 'Friends'
                        }
                        }},
                        {'item': {
                            '@jid': 'mercutio@example.com',
                            '@subscription': 'from'
                            }},
                            {'item': {
                                '@jid': 'benvolio@example.net',
                                '@subscription': 'both'
                                }},
                                {'item': {
                                    '@jid': 'foo@bar.baz',
                                    '@ask': 'subscribe',
                                    '@approved': 'true'
                                    }}
                                    ]
                                }
                                }}, stanza);
                            },
    
    'if there are vcards, it sends vcards in the roster': function() {
        var state = rosterFetchedState();
        var romeo_vcard = _('romeo_vcard').createStanza();
        state._stateData['vcards'] = {'romeo@example.net': romeo_vcard};
        var response = state.responseStanzas()[0];
        var romeo_entry = _(response.children[0].children).find(function(c){
            return c.attrs['jid'] === 'romeo@example.net'; 
        });
        assert.stanzaEqual({'vCard': {
            '@xmlns': 'vcard-temp',
            '#children': [
                {'FN': {
                    '#text': 'JeremieMiller'
                    },
                },
                {'N': {
                    'GIVEN': {
                        '#text': 'Jeremie'
                    },
                    'FAMILY': {
                        '#text': 'Miller'
                    },
                    'MIDDLE': {},
                    }
                },
                {'NICKNAME': {
                        '#text': 'jer'
                    },
                },
                {'EMAIL': {
                    'INTERNET': {},
                    'PREF': {},
                    'USERID': {
                        '#text': 'jeremie@jabber.org'
                        }
                    }
                },
                {'JABBERID': {
                    '#text': 'jer@jabber.org'
                    }
                }
            ]
        }}, _(romeo_entry.children).find(function(c){
            return c.name === 'vCard';
        }));
    }
}