var assert = require('assert');
var _ = require('underscore'); 
var dutil = require('node-xmpp-bosh').dutil;
var fs = require('fs');


_.mixin({
    xml_parse_file: function(filename) {
        if (filename === 'dummy') return dutil.xml_parse("<dummy/>");
        return dutil.xml_parse(fs.readFileSync(__dirname + '/packets/' + filename, 'utf8').
        replace(/\n|\t|^\s*|\s(?=\s)|\s*$/g,""));
    },
    createStanza: function(type){
        return _(type).xml_parse_file();
    },
    createNodes: function(type){
        return _(type).xml_parse_file().children;
    },
    to_be_implemented: function() {
        assert.fail('to be implemented');
    }
});

assert.isFalse = function(value){
	assert.equal(false, value);	
}

assert.isTrue = function(value) {
    assert.equal(true, value);
}

assert.stanzaEqual = function(json, node) {
    var nodeName = _(json).chain().keys().first().value();
    var nodeData = _(json).chain().values().first().value();
    if (json == {}) return true;
    assert.equal(nodeName, node.name);

    _(nodeData).chain().keys().map(function(k){
        return /^@(.*)/.exec(k)
    }).select(function(match){
        return match
    }).each(function(match){
        assert.equal(nodeData[match[0]], node.attrs[match[1]], 'unmatched attribute value of attribute ' + match[1] + ' for ' + nodeName + '. Got ' + node.attrs[match[1]] + ' instead of ' + nodeData[match[0]]);
    });

    _(nodeData).chain().keys().select(function(k){
        return !(/^(?:@|#)(.*)/.test(k))
    }).each(function(key){
        var nextJson = {}
        nextJson[key] = nodeData[key];
        var nextNode = _(node.children).find(function(c){
             return c.name === key
        });
        assert.isDefined(nextNode, "no child named " + key + " in " + nodeName)
        assert.stanzaEqual(nextJson, nextNode);
    })

    if ('#text' in nodeData) {
        assert.equal(nodeData['#text'], node.getText());
    }
    
    if ('#children' in nodeData) {
        _(nodeData['#children']).chain().zip(node.children).each(function(childSet){
            assert.isDefined(childSet[1], "missing child " + _(childSet[0]).head());
            assert.stanzaEqual(childSet[0], childSet[1]);
        });
    }
}
